<?php
/**
Utility functions used by PrivateWiki extension.

Contains a variety of functions used to help make private wikis more private.
**/
class PrivateWiki {
   # NoRememberPrefHook -- don't display "remember password" checkbox.
   #
   # Prevents display of the "remember password" checkbox on the login screen.
   # This function can be hung on the GetPreferences hook to prevent display
   # of the "remember password" checkbox on the login screen.
   static function NoRememberPrefHook($user, &$preferences) {
      unset($preferences['rememberpassword']);
      return true;
   }

   # RecordLoginFail -- write to log file on login failure (for fail2ban)
   #
   # On login failure, write a line to a log file. The log file format
   # is compatible with fail2ban.
   public static function RecordLoginFail($response, $user, $username) {
      global $wgSitename;
      global $wgPrivateWikiLoginFailDir;
      if(empty($wgPrivateWikiLoginFailDir)) return true;
      if($response->status == "PASS") return true; # no-op on success
      $time = gmdate("[Y-m-d H:i:s]"); // [2017-09-25 18:51:29]
      $ip = $_SERVER['REMOTE_ADDR']; // actual client IP even if proxied
      $file = $wgPrivateWikiLoginFailDir."/".gmdate("Y-m-d").".log";
      error_log("$time auth fail site=$wgSitename ip=$ip\n", 3, $file);
      return true; # fall through to next hook
   }

   # NoAutocomplete -- turn off remember-password / wallet browser features
   #
   # This method can be hung on the BeforePageDisplay hook to keep the
   # browser from offering to "remember" (i.e., insecurely store) the
   # user's credentials on the Special:UserLogin page.
   #
   # What a total mess.
   #
   # Problem 1: Browser-native autocomplete is a bad idea to begin with,
   # and persists only because of historical inertia. (Pages that have a
   # use for it can achieve better results with JS -- see Google's search
   # suggestions for example.)
   #
   # Problem 2: Autocomplete is a genuinely horribe idea for any kind of
   # sensitive and/or password-equivalent data. (If the browser "remembers"
   # the password, what exactly are you protecting, and from whom? Why even
   # have a password at all?) If it *has* to exist, and *has* to be on by
   # default, then FFS at least disable it for non-echoing fields, and
   # fields whose name contains the case-insensitive substring "pass" (or
   # the locale-specific equivalent).
   #
   # Problem 3: Given that we want to turn it off, there's no clean way to
   # do that. The "autocomplete" attribute is in HTML5, and is widely
   # supported even outside of that (since you basically can't do online
   # banking or e-commerce without it). But it's not fully portable and
   # won't pass validation.
   #
   # Problem 4: Ok, we want to give the login form the autocomplete="no"
   # attribute. That lives in the includes/templates/Userlogin.php.
   # We can't just patch or replace the template from inside an extension
   # (or, at least, I don't know how). The UserLoginForm hook only lets
   # us mess with the parameters passed *to* the template, not the static
   # parts of the template itself. The only way I could see to do this is
   # to hook BeforePageOutput and edit the raw HTML for the entire page.
   #
   # Problem 5: BeforePageOutput gets called for *every* page, so that's
   # a nice performance hit (even if we avoid the expensive preg_replace()
   # on pages other than Special:UserLogin).
   #
   # Problem 6: This is going to break the instant someone changes the
   # template in a way that doesn't match the regexp.
   static function NoAutocomplete(OutputPage &$out, Skin &$skin) {
      # only do replacement if we're on Special:UserLogin
      if($skin->getTitle() != 'Special:UserLogin') return true;

      $out->mBodytext = preg_replace('/(<form[^>]+name="userlogin")/',
      '\1 autocomplete="no"', $out->mBodytext, -1, $count);

      return true;
   }
}
